import json
import os
import sys
import colored
import time
import shutil
import tempfile
from onepasswd import ltlog

log = ltlog.getLogger('onepasswd.tools.jmerge')


def log_diff(tag, op, color, key, v):
    print("%s [%s] [%s] {%s: %s} %s"
          % (colored.fg(color),
             tag,
             op,
             key,
             json.dumps(v),
             colored.attr('reset')))


def log_local_add(key, v):
    log_diff("L", "+", "green", key, v)


def log_local_del(key, v):
    log_diff("L", "-", "red", key, v)


def log_remote_add(key, v):
    log_diff("R", "+", "blue", key, v)


def log_remote_del(key, v):
    log_diff("R", "-", "yellow", key, v)


def merge(fa, fb):
    log.debug(fa + ' ' + fb)
    tmp_bck_file = os.path.join(tempfile.gettempdir(), os.path.basename(
        fa) + '.' + str(time.time()) + '.bck')
    print("you can find backup file here:", tmp_bck_file)
    shutil.copyfile(fa, tmp_bck_file)
    ja, jb = {}, {}
    with open(fa, "r") as fpa:
        ja = json.load(fpa)
    with open(fb, "r") as fpb:
        jb = json.load(fpb)

    for keya, vala in ja.items():
        if keya not in jb:
            log_remote_add(keya, vala)
        elif jb[keya] != vala:
            at = float(vala['time'])
            bt = float(jb[keya]['time'])
            if at < bt:
                log_local_del(keya, vala)
                log_local_add(keya, jb[keya])
                ja[keya] = jb[keya]
            else:
                log_remote_del(keya, jb[keya])
                log_remote_add(keya, vala)
    for keyb, valb in jb.items():
        if keyb not in ja:
            log_local_add(keyb, valb)
            ja[keyb] = valb
    with open(fa, "w") as fpa:
        json.dump(ja, fpa, indent=2)


def main():
    assert len(sys.argv) == 3
    merge(*sys.argv[1:])


if __name__ == "__main__":
    main()



import sys
import json
import shutil
import time
from onepasswd.onepasswd import DB


def upgrade(file):
    shutil.copyfile(file, '.' + file + '.' + str(time.time()) + '.bck.db')
    db = {}
    with open(file, "r") as fp:
        db = json.load(fp)
    new_db = {}
    for k, v in db.items():
        entry = [k]
        new_db[DB.cal_key(entry)] = {
            "entries": entry,
            "passwd": v['passwd'],
            "time": v['time']
        }
    with open(file, "w") as fp:
        json.dump(new_db, fp, indent=2)


def main():
    upgrade(sys.argv[1])


if __name__ == "__main__":
    main()



import logging
import pyclbr
import bisect

mod = None


class ModUtils:
    def __init__(self, mod):
        line2func = []
        try:
            mod = pyclbr.readmodule_ex(mod)
        except Exception:
            print("ltlog internal error!")
            return
        for classname, cls in mod.items():
            if isinstance(cls, pyclbr.Function):
                line2func.append((cls.lineno, "<no-class>", cls.name))
            elif isinstance(cls, pyclbr.Class):
                for methodname, start in cls.methods.items():
                    line2func.append((start, classname, methodname))
            else:
                continue
        self.line2func = line2func
        self.line2func.sort()
        keys = [item[0] for item in self.line2func]
        self.keys = keys

    def line_to_class(self, lineno):
        index = bisect.bisect(self.keys, lineno) - 1
        if index < 0:
            return '<main>'
        return self.line2func[index][1]


class LtLogRecord(logging.LogRecord):
    def __init__(self, name, level, pathname, lineno, msg, args, exc_info, func, sinfo):
        super().__init__(name, level, pathname, lineno, msg, args, exc_info, func, sinfo)
        self.className = mod.line_to_class(self.lineno)


def getLogger(_mod):
    global mod
    mod = ModUtils(_mod)
    FORMAT = '[%(asctime)-15s] %(levelname)5s %(module)s | %(className)s.%(funcName)s():%(lineno)d -> %(message)s'
    logging.basicConfig(format=FORMAT)
    logger = logging.getLogger(_mod)
    logging.setLogRecordFactory(LtLogRecord)
    return logger



import onepasswd
from onepasswd.onepasswd import PasswdGen
from onepasswd.config import Config
from onepasswd import crypto
from onepasswd.onepasswd import DB
from onepasswd.onepasswd import GitDB
from onepasswd.tools import jmerge, jdiff
from onepasswd import ltlog
import pyperclip
from pathlib import Path
import os
import json
import shlex
import time
import tempfile
import click
import logging

log = ltlog.getLogger('onepasswd.client')

DEF_CONFIG_PATH = os.path.join(Path.home(), ".onepasswd", "onepasswd.json")
DEF_DB_PATH = os.path.join(Path.home(), ".onepasswd", "db")
DEF_BACKUP_PATH = os.path.join(Path.home(), ".onepasswd", "backup")


@click.group()
@click.option('-d', '--debug', 'debug', default='INFO', help='log level')
@click.option('-c', '--config', 'config', default=DEF_CONFIG_PATH, help='config path')
@click.option('-p', '--print_passwd', 'print_passwd', is_flag=True,
              help='print password (do not copy from clipboard)')
@click.pass_context
def cli(ctx: click.Context, debug, config, print_passwd):
    ctx.ensure_object(dict)
    logging.getLogger().setLevel(debug)
    ctx.obj['config_path'] = config
    if os.path.exists(config):
        ctx.obj['config'] = Config(config)
    if not is_clipboard_available():
        log.warn('clipboard is not available')
    ctx.obj['print_passwd'] = print_passwd


@cli.command('version')
@click.pass_context
def version(ctx: click.Context):
    click.echo(onepasswd.get_version())


@cli.command('init')
@click.option('-r', '--repo', 'repo', help='github repo (owner/name), used to upload database to github')
@click.option('-t', '--token', 'token', help='github token ("github_xxx")')
@click.option('-t', '--encrypted-token', 'encrypted_token', help='github token (encrypted)')
@click.option('--input-token', 'input_token', is_flag=True, help='get token from stdin')
@click.option('--db', 'db', default=DEF_DB_PATH, show_default=True, help='local database path')
@click.option('--backup', 'backup', default=DEF_BACKUP_PATH, show_default=True, help='database backup path')
@click.pass_context
def init(ctx: click.Context, repo, token, db, backup, input_token, encrypted_token):
    cfg_path = from_ctx(ctx, 'config_path')
    if os.path.exists(cfg_path):
        click.echo(f'already initialized. if you want to re-init, please remove {cfg_path}')
        exit(0)
    prepare_dir(cfg_path)
    prepare_dir(db)
    prepare_dir(backup)
    cfg = {
        'db': db,
        'backup': backup
    }
    # init auth info
    passwd = get_passwd(ctx, promt='master password', confirm=True,
                        from_clipboard=False, key='passwd')
    auth = crypto.generate_auth_info(passwd)
    cfg['auth_info'] = auth
    if repo and (token or input_token or encrypted_token):
        # init github
        cfg['repo'] = repo
        if input_token:
            token = get_passwd(ctx, promt='token')
        if not encrypted_token:
            encrypted_token = crypto.encrypt_passwd(token, passwd)
        cfg['token'] = encrypted_token
        # sync from github
        if click.confirm('sync from github'):
            # todo: sync
            ctx.obj['config'] = cfg
            ctx.invoke(sync)
    else:
        click.echo('github sync not enabled')
    # write config
    with open(cfg_path, "w") as fp:
        json.dump(cfg, fp, indent=2)
    click.echo("init finish!")


@cli.command('sync')
@click.pass_context
def sync(ctx: click.Context):
    cfg = from_ctx(ctx, 'config')
    assert cfg is not None
    passwd = get_passwd(ctx, promt='master password', from_clipboard=False, key='passwd')
    token = crypto.decrypt_passwd(cfg['token'], passwd)
    git = GitDB(cfg['repo'], token)
    remote_sha = git.get_remote_hash('db')
    if git.hashfile(cfg['db']) == remote_sha:
        click.echo("local db is already updated")
        return
    # download and merge
    remote_db = git.pull('db')
    log.debug('get remote db +++ ' + str(remote_db))
    with tempfile.TemporaryDirectory(prefix='onepasswd') as dir:
        tmp_file = os.path.join(dir, 'db')
        with open(tmp_file, 'wb') as fp:
            fp.write(remote_db)
        click.echo("> diff >>>")
        jdiff.diff(cfg['db'], tmp_file)
        click.echo("> merge >>>")
        jmerge.merge(cfg['db'], tmp_file)
    # push
    git.push(cfg['db'], 'db', remote_sha)
    click.echo("sync success")


@cli.command('get')
@click.argument('keywords', nargs=-1)
@click.pass_context
def get(ctx: click.Context, keywords: tuple):
    if len(keywords) == 0:
        keywords = ('',)
    cfg = from_ctx(ctx, 'config')
    db = get_db(ctx)

    def process_passwd(passwd_enc):
        passwd = get_passwd(ctx, promt='master password', from_clipboard=False, key='passwd')
        if not crypto.auth(cfg['auth_info'], passwd):
            click.echo("password wrong")
            exit(1)
        give_passwd(ctx, crypto.decrypt_passwd(passwd_enc, passwd))

    # accurate match
    if keywords in db:
        item = db.get_item(keywords)
        click.echo(passwd_item_str(item))
        process_passwd(item['passwd'])
        return

    for v in db.find(keywords):
        if click.confirm(passwd_item_str(v)):
            process_passwd(v['passwd'])
            break


@cli.command('list')
@click.argument('keywords', nargs=-1)
@click.pass_context
def list(ctx: click.Context, keywords: tuple):
    if len(keywords) == 0:
        keywords = ('',)
    db = get_db(ctx)
    for v in db.find(keywords):
        click.echo(passwd_item_str(v))


@cli.command('add')
@click.argument('entries', nargs=-1)
@click.option('-l', '--len', 'length', default=12, show_default=True, help='password length')
@click.option('-s', '--strength', 'strength', default='aA0$', show_default=True,
              help='password strength')
@click.pass_context
def add(ctx: click.Context, entries: tuple, length, strength):
    d = {}

    def getpass():
        passwd = PasswdGen.generate(length, pre_process_strength(strength))
        d['passwd'] = passwd
        return passwd
    if save_passwd(ctx, entries, getpass):
        give_passwd(ctx, d['passwd'])
        if click.confirm('sync ?'):
            ctx.invoke(sync)


@cli.command('save')
@click.argument('entries', nargs=-1)
@click.pass_context
def save(ctx: click.Context, entries: tuple):
    def getpass():
        return get_passwd(ctx, promt='new password', confirm=True)
    if save_passwd(ctx, entries, getpass):
        if click.confirm('sync ?'):
            ctx.invoke(sync)


@cli.command('del')
@click.argument('keywords', nargs=-1)
@click.pass_context
def delete(ctx: click.Context, keywords: tuple):
    if len(keywords) == 0:
        keywords = ('',)
    db = get_db(ctx)
    for v in db.find(keywords):
        if click.confirm(passwd_item_str(v)):
            input = click.prompt('input entries to confirm delete')
            entries = shlex.split(input)
            if entries == v['entries']:
                del db[entries]
                click.echo(f'{passwd_item_str(v)} deleted')
                break


@cli.command('dec')
@click.argument('cipher')
@click.pass_context
def dec(ctx: click.Context, cipher):
    passwd = get_passwd(ctx, from_clipboard=False)
    text = crypto.decrypt_passwd(cipher, passwd)
    give_passwd(ctx, text)


@cli.command('enc')
@click.option('-d', '--data', 'data')
@click.pass_context
def enc(ctx: click.Context, data):
    passwd = get_passwd(ctx, from_clipboard=False)
    if not data:
        data = get_passwd(ctx, promt='content to encrypt')
    cipher = crypto.encrypt_passwd(data, passwd)
    give_passwd(ctx, cipher)


@cli.command('web')
@click.option('-h', '--host', 'host', default='0.0.0.0')
@click.option('-p', '--port', 'port', default=10066)
@click.option('-d', '--debug', 'debug', is_flag=True, default=False)
@click.pass_context
def web(ctx, host, port, debug):
    try:
        from onepasswd import web
    except ImportError:
        click.echo('you must install flask if you want to use onepasswd web')
        click.echo('try: pip install flask')
        exit(1)
    web.init(from_ctx(ctx, 'config'))
    web.app.run(host=host, port=port, debug=debug)


def save_passwd(ctx, entries, getpass):
    assert len(entries) >= 1
    db = get_db(ctx)
    if entries in db:
        item = db.get_item(entries)
        if not click.confirm(
                f'{passwd_item_str(item)} already in database, update ?'):
            return False
        cfg = from_ctx(ctx, 'config')
        db.backup(cfg['backup'])
    passwd = get_passwd(ctx, promt='master password', from_clipboard=False, key='passwd')
    passwd_dec = getpass()
    passwd_enc = crypto.encrypt_passwd(passwd_dec, passwd)
    db[entries] = passwd_enc
    return True


def pre_process_strength(s):
    strength = set()
    for x in s:
        for i, t in enumerate(PasswdGen.passwd_table):
            if x in t:
                strength.add(i)
                break
    return strength


def passwd_item_str(item):
    t = time.ctime(float(item['time']))
    entries = ' '.join(['"' + x.replace('"', '\\"') + '"' for x in item['entries']])
    return f'[{t}] {entries}'


def get_db(ctx: click.Context):
    config = from_ctx(ctx, 'config')
    return DB(config['db'])


def from_ctx(ctx: click.Context, name, default=None):
    return ctx.obj.get(name, default)


def check_config(cfg: dict):
    keys = ['db', 'backup']
    for key in keys:
        if key not in cfg:
            return False
    return True


def prepare_dir(filepath):
    dir = os.path.dirname(filepath)
    if not os.path.exists(dir):
        os.makedirs(dir)


def get_passwd(ctx: click.Context = None, promt="password", from_clipboard=None, confirm=False, key=None):
    def ret_password(passwd):
        if ctx and key and key not in ctx.obj:
            ctx.obj[key] = passwd
        return passwd
    if ctx and key and key in ctx.obj:
        return from_ctx(ctx, key)
    if (from_clipboard is None and click.confirm(f'[{promt}] use input from clipboard ?')) \
            or from_clipboard:
        passwd = pyperclip.paste().strip()
        if len(passwd) == 0:
            click.echo('your clipboard contains northing')
        else:
            return ret_password(passwd)
    passwd = click.prompt(f'please input {promt}', hide_input=True)
    passwd = passwd.strip()
    if confirm:
        confirm = click.prompt(f'confirm {promt}', hide_input=True)
        confirm = confirm.strip()
        if confirm != passwd:
            return get_passwd(ctx, promt=promt, from_clipboard=False, confirm=True, key=key)
    return ret_password(passwd)


def give_passwd(ctx: click.Context, passwd, no_echo=False):
    if not from_ctx(ctx, 'print_passwd') and is_clipboard_available():
        pyperclip.copy(passwd)
    else:
        if not no_echo:
            click.echo(passwd)


def is_clipboard_available():
    try:
        pyperclip.paste()
        return True
    except Exception as e:
        log.error(e)
        return False


def main():
    cli()


if __name__ == "__main__":
    main()



from flask import Flask
from flask import render_template
from flask import request, make_response
from onepasswd import crypto
from onepasswd import ltlog
from onepasswd.onepasswd import DB
import datetime
import os
from Crypto.PublicKey import RSA
import base64
from Crypto.Cipher import PKCS1_v1_5

log = ltlog.getLogger('onepasswd.web')

_key = RSA.generate(2048)
_public_key = _key.public_key().export_key(format='PEM').decode()
_db_path = ''
_web_path = os.path.abspath(
    os.path.join(os.path.dirname(__file__), 'templates'))
app = Flask("onepasswd", template_folder=_web_path,
            static_folder=os.path.join(_web_path, 'static'))


def init(cfg):
    global _db_path
    _db_path = cfg['db']


@app.route("/")
def index():
    _db = DB(_db_path)
    items = []
    for entry in _db.getkeys():
        v = _db.get_item(entry)
        items.append(v)
    items = sorted(items, key=lambda v: float(v['time']), reverse=True)
    for i, v in enumerate(items):
        items[i] = {
            'time': str(datetime.datetime.fromtimestamp(int(float(v['time'])))),
            'entry': v['entries'],
            'data': v['passwd']
        }
    return render_template('index.html', items=items, pubkey=_public_key)


@app.post("/decrypt")
def decrypt():
    resp_500 = make_response()
    resp_500.status_code = 500

    data = request.json['data']
    passwd = request.json['passwd']
    log.debug(request.json)
    decryptor = PKCS1_v1_5.new(_key)
    passwd = decryptor.decrypt(base64.b64decode(passwd.encode()), None).decode()
    if not passwd:
        return resp_500
    try:
        text = crypto.decrypt_passwd(data, passwd)
    except:
        return resp_500
    return {'data': text}



#!/usr/bin/python3

import json
import random
import time
import os
import socket
from onepasswd import ltlog
from hashlib import sha256, sha1
from github import Github

log = ltlog.getLogger('onepasswd.onepasswd')

g_req_kwarg = {
    # 'verify': False
}


class GitDB(object):
    def __init__(self, repo, token):
        super().__init__()
        self.github = Github(token)
        self.token = token
        log.debug(f'get_repo {repo}')
        self.repo = self.github.get_repo(repo)

    def _gen_commit_msg(self):
        localname = socket.gethostname()
        create_time = time.ctime()
        return "[{time:}] @ {name}".format(time=create_time, name=localname)

    def pull(self, path):
        file = self.repo.get_contents(path)
        return file.decoded_content

    def push(self, local_path, remote_path, sha=None):
        with open(local_path) as fp:
            buf = fp.read()
        msg = self._gen_commit_msg()
        if sha:
            self.repo.update_file(remote_path, msg, buf, sha)
        else:
            self.repo.create_file(remote_path, msg, buf)

    def get_remote_hash(self, path):
        dir, name = os.path.split(path)
        files = self.repo.get_dir_contents(dir)
        sha = None
        for f in files:
            if f.name == name:
                sha = f.sha
        return sha

    @staticmethod
    def hashfile(file):
        res = None
        with open(file, "rb") as fp:
            length = fp.seek(0, os.SEEK_END)
            fp.seek(0, os.SEEK_SET)
            header = b'blob ' + str(length).encode() + b'\x00'
            res = sha1(header + fp.read()).hexdigest().lower()
        return res


class PasswdGen(object):
    passwd_table = ["abcdefghijklmnopqrstuvwxyz",
                    "ABCDEFGHIJKLMNOPQRSTUVWXYZ",
                    "0123456789",
                    "@_-~,.#$"]

    @staticmethod
    def check_strength(passwd, strength):
        vis = set()
        for x in passwd:
            for i, t in enumerate(PasswdGen.passwd_table):
                if x in t:
                    vis.add(i)
        if vis != strength:
            return False
        return True

    @staticmethod
    def generate(length=12, strength=set(range(0, len(passwd_table)))):
        table = ''.join([PasswdGen.passwd_table[i] for i in strength])
        passwd = ''
        for i in range(length):
            passwd += random.choice(table)

        if PasswdGen.check_strength(passwd, strength):
            return passwd
        else:
            return PasswdGen.generate(length, strength)


class DB(dict):
    def __init__(self, path, default={}):
        self.path = path
        self.db = default
        self._load_from_disk()

    def __getitem__(self, entries):
        key = DB.cal_key(entries)
        return self.db[key]['passwd']

    @staticmethod
    def cal_key(entries):
        return sha256(json.dumps(entries).replace('\n', '').replace('\t', '').encode('utf-8')).hexdigest()

    def __setitem__(self, entries, val):
        key = DB.cal_key(entries)
        if key in self.db and self.db[key]['passwd'] == val:
            return
        self.db[key] = {'entries': entries,
                        'passwd': val, 'time': str(time.time())}
        self._write_to_disk()

    def __delitem__(self, entries):
        key = DB.cal_key(entries)
        if key not in self.db:
            return
        self.db.__delitem__(key)
        self._write_to_disk()

    def __contains__(self, entries):
        return self.db.__contains__(DB.cal_key(entries))

    def _write_to_disk(self, path=None):
        path = path if path else self.path
        with open(path, 'w') as fp:
            json.dump(self.db, fp, indent=2)

    def _load_from_disk(self, path=None, default={}):
        path = path if path else self.path
        if not os.path.exists(path):
            self._write_to_disk(path)
        with open(path) as fp:
            self.db = json.load(fp)

    def backup(self, backup_path):
        if not os.path.exists(backup_path):
            os.makedirs(backup_path)
        bck_file = os.path.join(backup_path, str(time.time()) + '.db')
        self._write_to_disk(bck_file)

    def rollback(self, timestamp, backup_path):
        timestamp = str(timestamp)
        bck_file = os.path.join(backup_path, timestamp + '.db')
        if not os.path.exists(bck_file):
            print("[Abort] '{}' not found".format(bck_file))
            return
        # do backup
        self.backup()
        self._load_from_disk(bck_file)

    def getkeys(self):
        return [x['entries'] for k, x in self.db.items()]

    def _find(self, keyword):
        s = set()
        for h, v in self.db.items():
            for e in v['entries']:
                if keyword in e:
                    s.add(h)
                    break
        return s

    def find(self, keywords):
        r = []
        for key in keywords:
            s = self._find(key)
            if len(r) == 0:
                r = s
            else:
                r.intersection_update(s)
            if len(r) == 0:
                return []
        return [self.db[x] for x in r]

    def get_item(self, entries):
        key = DB.cal_key(entries)
        return self.db[key]


#!/usr/bin/python3

import json
import sys
import colored


def diff(fa, fb):
    fpa = open(fa)
    fpb = open(fb)

    ja = json.load(fpa)
    jb = json.load(fpb)

    for keya, vala in ja.items():
        if keya not in jb:
            print("%s [A] {%s: %s} %s"
                  % (colored.fg('red'),
                     keya, json.dumps(vala, indent=4),
                     colored.attr('reset')))
        elif jb[keya] != vala:
            print("%s [A] {%s: %s} %s"
                  % (colored.fg('red'),
                     keya, json.dumps(jb[keya], indent=4),
                     colored.attr('reset')))
            print("%s [B] {%s: %s} %s"
                  % (colored.fg('green'),
                     keya, json.dumps(vala, indent=4),
                     colored.attr('reset')))

    for keyb, valb in jb.items():
        if keyb not in ja:
            print("%s [B] {%s: %s} %s"
                  % (colored.fg('green'),
                     keyb, json.dumps(valb),
                     colored.attr('reset')))
    fpa.close()
    fpb.close()


def main():
    assert len(sys.argv) == 3
    diff(*sys.argv[1:])


if __name__ == "__main__":
    main()



import setuptools

setuptools.setup(
    name="onepasswd",
    use_scm_version=True,
    author="agfn",
    author_email="lavender.tree9988@gmail.com",
    description="onepasswd",
    long_description='onepasswd',
    long_description_content_type="text/markdown",
    setup_requires=['setuptools_scm'],
    install_requires=[
        'setuptools_scm',
        'pycryptodome',
        'pyperclip',
        'requests',
        'colored',
        'click',
        'PyGithub'
    ],
    url="https://github.com/agfn/onepasswd",
    project_urls={
        "Bug Tracker": "https://github.com/agfn/onepasswd/issues",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    packages=setuptools.find_packages(),
    python_requires=">=3.6",
    entry_points='''
    [console_scripts]
        onepasswd=onepasswd.client:main
        onepasswd-upgrade=onepasswd.tools.upgrade:main
        onepasswd-merge=onepasswd.tools.jmerge:main
        onepasswd-diff=onepasswd.tools.jdiff:main
    ''',
    include_package_data=True,
    package_data={"onepasswd": ["templates/*.html", "templates/static/*"]}
)



import os
from pathlib import Path
import json
from onepasswd import ltlog

log = ltlog.getLogger('onepasswd.config')


class Config(object):
    CONF_PATH = os.path.join(Path.home(), '.onepasswd', 'onepasswd.conf')

    def __init__(self, conf_path=CONF_PATH):
        self.conf_path = conf_path
        if not os.path.exists(conf_path):
            with open(conf_path, "w") as fp:
                fp.write("{}")
        fp = open(conf_path, "r")
        self.conf = json.load(fp)
        fp.close()

    def __getitem__(self, x):
        return self.conf[x]

    def __setitem__(self, k, v):
        if k not in self.conf or (k in self.conf and v != self.conf[k]):
            self.conf[k] = v
            self._write_to_disk()

    def _write_to_disk(self):
        with open(self.conf_path, "w") as fp:
            log.debug(self.conf)
            json.dump(self.conf, fp, indent=4)




def get_version(default='version not found'):
    try:
        from pkg_resources import DistributionNotFound, get_distribution
    except ImportError:
        return default
    else:
        try:
            return get_distribution(__name__).version
        except DistributionNotFound:
            return default



from Crypto.Cipher import AES
import binascii
from onepasswd import ltlog
from Crypto.PublicKey import ECC
from hashlib import sha256

log = ltlog.getLogger('onepasswd.crypto')


class WrongPasswordException(Exception):
    pass


def hash_key(key):
    return sha256(sha256(key.encode()).hexdigest().encode()).hexdigest()


def decrypt_passwd(passwd_enc, key):
    key = binascii.unhexlify(hash_key(key))
    passwd_enc = binascii.unhexlify(passwd_enc)
    cipher = AES.new(key, AES.MODE_ECB)
    passwd = cipher.decrypt(passwd_enc)
    return passwd.decode().strip()


def encrypt_passwd(passwd, key):
    key = binascii.unhexlify(hash_key(key))
    passwd = passwd.encode()
    passwd = passwd.ljust(16 * (((len(passwd) - 1) // 16) + 1), b' ')
    cipher = AES.new(key, AES.MODE_ECB)
    passwd_enc = cipher.encrypt(passwd)
    return binascii.hexlify(passwd_enc).decode()


def generate_auth_info(passwd):
    key = ECC.generate(curve='P-256')
    auth_info = {
        'pub': key.public_key().export_key(format='PEM'),
        'priv': key.export_key(passphrase=passwd,
                               protection='PBKDF2WithHMAC-SHA1AndAES128-CBC',
                               format='PEM')
    }
    log.debug(auth_info)
    return auth_info


def auth(auth_info, passwd):
    try:
        priv = ECC.import_key(auth_info['priv'], passphrase=passwd)
    except Exception:
        return False
    pub = ECC.import_key(auth_info['pub'])
    result = priv.public_key() == pub
    log.debug(result)
    return result
